<?php

declare(strict_types=1);

namespace FiloBlu\Checkproducts\Model;

use FiloBlu\Checkproducts\Helper\Data;
use FiloBlu\Checkproducts\Helper\Email;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Eav\Model\ResourceModel\Entity\Attribute;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\StoreManagerInterface;
use const Magento\Framework;

/**
 *
 */
class ProductChecker implements ProductCheckerInterface
{

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Email
     */
    private $emailHelper;

    /**
     * @var CollectionFactory
     */
    private $productCollectionFactory;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var CollectorInterfaceFactory
     */
    private $collectorFactory;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var Attribute
     */
    private $eavAttribute;
    /**
     * @var MetadataPool
     */
    private $metadataPool;

    /**
     * @param CollectionFactory $productCollectionFactory
     * @param CollectorInterfaceFactory $collectorFactory
     * @param Data $helper
     * @param Email $emailHelper
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface $scopeConfig
     * @param ResourceConnection $resourceConnection
     * @param Attribute $eavAttribute
     * @param MetadataPool $metadataPool
     */
    public function __construct(
        CollectionFactory $productCollectionFactory ,
        CollectorInterfaceFactory $collectorFactory ,
        Data $helper ,
        Email $emailHelper ,
        StoreManagerInterface $storeManager ,
        ScopeConfigInterface $scopeConfig ,
        ResourceConnection $resourceConnection ,
        Attribute $eavAttribute ,
        MetadataPool $metadataPool


    )
    {
        $this->scopeConfig = $scopeConfig;
        $this->helper = $helper;
        $this->emailHelper = $emailHelper;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->storeManager = $storeManager;
        $this->collectorFactory = $collectorFactory;
        $this->resourceConnection = $resourceConnection;
        $this->eavAttribute = $eavAttribute;
        $this->metadataPool = $metadataPool;
    }

    /**
     * @param CollectorInterface $collector
     * @return CollectorInterface
     * @throws LocalizedException
     */
    public function check(CollectorInterface $collector): CollectorInterface
    {
        if (!$this->helper->isEnabled()) {
            return $collector;
        }

        foreach ($this->storeManager->getStores(true) as $store) {
            $this->checkProducts($store , $collector);
        }

        return $collector;
    }

    /**
     * @param StoreInterface $store
     * @param CollectorInterface $collector
     * @return void
     * @throws LocalizedException
     */
    public function checkProducts(StoreInterface $store , CollectorInterface $collector)
    {
        $metadata = $this->metadataPool->getMetadata(ProductInterface::class);
        $linkField = $metadata->getLinkField();
        $toEnable = [];
        $toDisable = [];

        foreach ($this->createCollection($store) as $product) {
            /** @var Product $product */
            $valid = $this->validate($product , $collector);

            if ((int)$product->getData('manual_status')) {
                continue;
            }

            if ($valid) {
                $toEnable[] = $product->getData($linkField);
                continue;
            }

            $toDisable[] = $product->getData($linkField);
        }

        if (!empty($toEnable)) {
            $this->updateAttributes(
                $toEnable ,
                Status::STATUS_ENABLED ,
                $linkField ,
                $store->getId()
            );
        }

        if (!empty($toDisable)) {
            $this->updateAttributes(
                $toDisable ,
                Status::STATUS_DISABLED ,
                $linkField ,
                $store->getId()
            );
        }
    }

    /**
     * @param StoreInterface $store
     * @return Collection
     * @throws LocalizedException
     */
    public function createCollection(StoreInterface $store)
    {
        /** @var Collection $productCollection */
        $productCollection = $this->productCollectionFactory->create();
        $productCollection->setStore($store);

        $productCollection->addAttributeToSelect(
            array_merge(
                $this->helper->getAllAttributesToCheck() ,
                ['price' , 'status' , 'to_check' , 'manual_status']
            )
        );

        foreach ($this->helper->getAttributeCheckAllowedTypes() as $allowedType) {
            if ($this->helper->canCheckImage($allowedType)) {
                $productCollection->addMediaGalleryData();
            }

            if ($this->helper->canCheckCategories($allowedType)) {
                $productCollection->addCategoryIds();
            }
        }

        $productCollection->addFieldToFilter(
            [
                ['attribute' => 'manual_status' , 'neq' => 1] ,
                ['attribute' => 'to_check' , 'eq' => 1]
            ]
        );

        return $productCollection;
    }

    /**
     * @param Product|ProductInterface $product
     * @param CollectorInterface $collector
     * @return bool
     * @throws LocalizedException
     */
    public function validate(ProductInterface $product , CollectorInterface $collector): bool
    {
        $type = $product->getTypeId();

        $canCollect = (bool)$product->getData('to_check');
        $valid = true;

        if ($this->helper->canCheckPrice($type) && $this->helper->getPrice($product) <= 0.0) {
            if (!$canCollect) {
                return false;
            }

            $valid = false;
            $collector->collect($product , $type === 'configurable' ? 'price_configurable' : 'price');
        }

        if ($this->helper->canCheckImage($type) && count($product->getMediaGalleryEntries()) === 0) {
            if (!$canCollect) {
                return false;
            }

            $valid = false;
            $collector->collect($product , 'images');
        }

        if ($this->helper->canCheckCategories($type) && count($product->getCategoryIds()) === 0) {
            if (!$canCollect) {
                return false;
            }

            $valid = false;
            $collector->collect($product , 'categories');
        }

        foreach ($this->helper->getAttributesToCheck($type) as $attributeCode) {
            if (!$product->getData($attributeCode)) {
                if (!$canCollect) {
                    return false;
                }

                $valid = false;
                $collector->collect($product , [$attributeCode => null]);
            }
        }

        return $valid;
    }

    /**
     * @param array $toEnable
     * @param $status
     * @param $linkField
     * @param int $storeId
     * @return void
     */
    private function updateAttributes(array $toEnable , $status , $linkField , $storeId)
    {
        $connection = $this->resourceConnection->getConnection();

        $attributeId = $this->eavAttribute->getIdByCode(
            Product::ENTITY ,
            ProductInterface::STATUS
        );

        $mainTable = $connection->getTableName('catalog_product_entity_int');

        $data = [];

        foreach ($toEnable as $id) {
            $data[] = [
                $linkField => $id ,
                'store_id' => $storeId ,
                'value' => $status ,
                'attribute_id' => $attributeId
            ];
        }

        $connection->insertOnDuplicate(
            $mainTable ,
            $data ,
            [
                'value'
            ]
        );
    }
}
